var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
var defaults = {
    decimalSeparator: ',',
    thousandSeparator: ' ',
    precision: 2,
    allowNegativeValues: false,
};
export function createFormattedNumberInput(InputComponent, options) {
    if (options === void 0) { options = {}; }
    var opts = __assign(__assign({}, defaults), options);
    var parse = function (value) {
        if (value) {
            var cleaned = value
                .replace(/\s/g, '')
                .replace(new RegExp(opts.decimalSeparator), '.');
            var number = parseFloat(cleaned);
            return !isNaN(number) ? number : undefined;
        }
    };
    var format = function (value) {
        value = value.replace(opts.allowNegativeValues ? /[^\d.,-]/g : /[^\d.,]/g, '');
        // only keep the first decimal separator
        value = value
            .replace(/[.,]/, '_')
            .replace(/[.,]/g, '')
            .replace(/_/, opts.decimalSeparator);
        // only keep `opts.precision` fraction digits
        if (value.indexOf(opts.decimalSeparator) !== -1) {
            var _a = value.split(opts.decimalSeparator), integer = _a[0], fractional = _a[1];
            value = integer + opts.decimalSeparator + fractional.substr(0, opts.precision);
        }
        // separate thousands
        value = value.replace(/\B(?=(\d{3})+(?!\d))/g, opts.thousandSeparator);
        return value;
    };
    return /** @class */ (function (_super) {
        __extends(FormattedNumberInput, _super);
        function FormattedNumberInput() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.ref = React.createRef();
            _this.caretPosition = 0;
            _this.state = { formattedValue: '' };
            _this.handleChange = function (event) {
                var inputted = event.currentTarget.value;
                var formatted = format(inputted);
                var parsed = parse(formatted);
                var delta = formatted.length - inputted.length;
                _this.caretPosition = _this.ref.current && _this.ref.current.selectionEnd
                    ? Math.max(_this.ref.current.selectionEnd + delta, 0)
                    : 0;
                _this.setState({ formattedValue: formatted }, function () {
                    _this.props.onChange(parsed);
                });
            };
            return _this;
        }
        FormattedNumberInput.getDerivedStateFromProps = function (nextProps, prevState) {
            var formattedValue = format(String(nextProps.value));
            var prevFormattedValueWithoutSpecialCharacters = prevState.formattedValue
                .replace(new RegExp(opts.decimalSeparator + "0$"), '')
                .replace(new RegExp("[" + opts.decimalSeparator + "-]"), '');
            if (formattedValue !== prevFormattedValueWithoutSpecialCharacters) {
                return { formattedValue: formattedValue };
            }
            return null;
        };
        FormattedNumberInput.prototype.componentDidUpdate = function (prevProps) {
            if (this.ref.current && prevProps.value !== this.props.value) {
                this.ref.current.setSelectionRange(this.caretPosition, this.caretPosition);
            }
        };
        FormattedNumberInput.prototype.render = function () {
            return (React.createElement(InputComponent, __assign({}, this.props, { ref: this.ref, value: this.state.formattedValue, onChange: this.handleChange })));
        };
        return FormattedNumberInput;
    }(React.Component));
}
;
//# sourceMappingURL=index.js.map